package Aww::Main;

use strict;
use warnings;
use Aww::lib qw(admininput checkoptions checksite config execoptions getsubdirs optioninput print_error trashfiles wmlprocess);

use Data::Dumper;
use Crypt::CBC ;


=pod

=head1 NAME

B<Aww::Main> - Perl Module for aww

=head1 DESCRIPTION

This manpage describes shortly all functions of the B<Aww::Main>
module.

=cut

####################################
# Subroutines processing the options

=pod

=head2 newsite subroutine

This subroutine ask for input to create an new aww project and
stores everything in the project file: <name>.aww

=cut


# newsite subroutine:

sub newsite {
    print "-- executing newsite subroutine --\n" if $Aww::verbose > 1 ;
    shift;
    my @CreateSites = @_;
    print_error("You can't make 'all' sites new!\n\n") if ( grep { /^all$/ } @CreateSites );


    my @ExistingSites = checksite(@_);
    if ( @ExistingSites ) {
	print_error("The Site(s) '@ExistingSites' already exist and can't be created!\n\n");
    }

    my $CreateSite;
    foreach $CreateSite (@CreateSites) {

	my $CreateFile = "${CreateSite}.aww" ;


	print "\nPlease answer the following questions to create\nan aww-project file for your '$CreateSite' website\n\n" ;


	# Ask for Export_FTP_Server:
	print "Please enter the FTP Server to Export the new Site to:\n" ;
	print "(only needed if you export your htmlfiles to an external server)\n > ftp://";
	admininput($CreateFile,"Export_FTP_Server");


	# Ask for Export_FTP_Directory:
	print "Please enter the export directory on this FTP Server:\n > ";
	admininput($CreateFile,"Export_FTP_Directory");


	# Ask for Export_FTP_Login:
	print "Please enter your login for this FTP Server:\n > ";
	admininput($CreateFile,"Export_FTP_Login");


	# Ask for Export_FTP_Password:
	print "Please enter your password for this FTP Server:\n";
	print "(The password is stored with very simple encryption in the\n";
	print " config file - leave blank if you don't want that)\n > ";

	admininput($CreateFile,"Export_FTP_Password");


	# Ask for Output_Base_Directory:
	print "Please enter the Base Directory for the Outputfiles:\n" ;
	print "(where to put the created htmlfiles)\n > ";
	admininput($CreateFile,"Output_Base_Directory");

        # Create Output_Base_Directory if not existent
	my $OBD = config($CreateFile,"Main.Output_Base_Directory") ;

	unless ( -d $OBD ) {	    
	    mkdir $OBD or print_error("Cannot create Directory: $OBD") ;
	}


	# Ask for Src_Base_Directory:
	print "Please enter the Base Directory of your Sourcefiles:\n" ;
	print "(where your .wml files reside)\n > ";
	admininput($CreateFile,"Src_Base_Directory");

	# Ask for recursive check of subdirectories
	print "Do you want to add all subdirectories? ([y]|n)\n > " ;
	my @AllSubDirs ;
	chomp ( my $SubDirAnswer = <STDIN> ) ;
	$SubDirAnswer = "y" unless $SubDirAnswer ;
	if ( $SubDirAnswer =~/^y/i ) {

	    # Checking subdirs of the Src_Base_Directory:
	    print "\nChecking for subdirectories...\n\n";
	    @AllSubDirs = getsubdirs(config($CreateFile,"Main.Src_Base_Directory"));

	}
	else {

	    @AllSubDirs = config($CreateFile,"Main.Src_Base_Directory") ;

	}


	my $AllSubDirs = "@AllSubDirs" ;
	$AllSubDirs =~ tr/ /,/ ;

	print $AllSubDirs ;

	config($CreateFile,"Main.ProcessDirectories",$AllSubDirs) ;


	# Ask for the options of every directory

	print "\nPlease give the options for the Directories:\n";
	print "(just hit return if there are no options)\n\n";

	foreach (@AllSubDirs) {
	    print "$_: ";
	    optioninput($CreateFile,$_);
	}



	print "\ncreated new project file: $ENV{HOME}/.aww/$CreateFile \n\n" if $Aww::verbose ;

    }

    return ;

}









# showsite subroutine:

=pod

=head2 showsite subroutine

This subroutine prints out all information about the given project!


=cut

sub showsite {
    shift;
    my @ShowSites = @_ ;

    print "-- executing showsite subroutine --\n" if $Aww::verbose > 1 ;

    my @ExistingSites = checksite(@_);
    print_error("The project(s) '@ShowSites' don't exist! You have to create a config file first.\n\n") unless @ExistingSites;

    foreach my $ShowSite ( @ExistingSites ) {
	print "\n--> Dumping internal structure of $ShowSite project:\n\n" ;
	my $ShowSite = "$ShowSite.aww" ;
	my $Configuration = config($ShowSite) ;

	print Dumper($Configuration) ;

    }

    return ;

}








# deletesite subroutine:

=pod

=head2 deletesite subroutine

This subroutine deletes the given project (after asking if you really
want it!)

=cut


sub deletesite {
    shift;
    print "-- executing deletesite subroutine --\n" if $Aww::verbose > 1 ;

    my @DeleteConf = checksite(@_);
    unless (@DeleteConf) {
	print "Config-files for '@_' do not exist and can't be deleted!\n";
	exit 0;
    }
    print "\n Do you really want to delete the complete configuration\n for the '@DeleteConf' Aww-Website(s)? (Y/[N])\n > ";
    foreach (@DeleteConf) {
	$_ = "$_.aww";
    }

    my $DelBool ;
    chomp ($DelBool = <STDIN>);
    if ( $DelBool =~ m/^y$/i ) {
	chdir "$ENV{HOME}/.aww/" ;
	unlink @DeleteConf;
	print "@DeleteConf configurations deleted! \n";
    }
    else {
	print "No configuration-files deleted!\n"; 
    }

    exit 0 ;

}







# ftpexportsite subroutine:

=pod

=head2 ftpexportsite subroutine

This subroutine exports the given aww-project with lftp (mirror) or
ncftpput to a external server. Check the .aww/config file to see which
ftp-program is used.

=cut


sub ftpexportsite {
    print "-- executing ftpexportsite subroutine --\n" if $Aww::verbose > 1 ;
    shift ;
    my @ExportSites = @_;

    my @ExistingSites = checksite(@_);
    print_error("The project(s) '@ExportSites' don't exist! You have to create a config file first.\n\n") unless @ExistingSites;


    foreach my $ExportSite (@ExistingSites) {
	my $ExportFile = "$ExportSite.aww" ;

	print "Exporting $ExportSite project:\n" if $Aww::verbose ;

	my $ExportProgram = config("config","Export_FTP_Program") ;
	my $ExportLogin = config($ExportFile,"Main.Export_FTP_Login") ;
	my $ExportCrypt = config($ExportFile,"Main.Export_FTP_Password") ;
	# decrypting password:
	my $ExportPassword ;
	if ( $ExportCrypt ) {
	    my $cp = Crypt::CBC->new( {'key'   => 'HaCKerZANzwer',
				       'cipher' => 'Blowfish' }) ;

	    $ExportPassword = $cp->decrypt($ExportCrypt) ;
	}

	my $ExportServer = config($ExportFile,"Main.Export_FTP_Server") ;
	my $ExportDirectory = config($ExportFile,"Main.Export_FTP_Directory") ;
	my $OutputDir = config($ExportFile,"Main.Output_Base_Directory") ;


	if ( $ExportProgram eq "lftp" ) {

	    $ExportCrypt = "$ExportLogin,<password>" ;
	    $ExportLogin = "$ExportLogin,$ExportPassword" if $ExportPassword ;


	    print qq-- executing: $ExportProgram -u $ExportCrypt $ExportServer -e "mirror -R $OutputDir $ExportDirectory;bye" \n if $Aww::verbose ;

	    exec qq{$ExportProgram -u $ExportLogin $ExportServer -e "mirror -R $OutputDir $ExportDirectory;bye"} or die "Problem during lftp export: $!\n" ;


	}
	elsif ( $ExportProgram eq "ncftpput" ) {

	    $ExportCrypt = "-u $ExportLogin -p <password>" ;
	    $ExportLogin = "-u $ExportLogin" ;
	    $ExportLogin = "$ExportLogin -p $ExportPassword" if $ExportPassword ;

	    print qq-- executing: $ExportProgram $ExportCrypt -R $ExportServer $ExportDirectory $OutputDir \n if $Aww::verbose ;

	    exec qq{$ExportProgram $ExportLogin -R $ExportServer $ExportDirectory $OutputDir/*} or die "Problem during ncftpput export: $!\n" ;


	}
	else {
	    print "FTP-program '$ExportProgram' not supported!\n" ;
	    print "Check your config-file! --> exiting.\n" ;
	    exit 1 ;
	}

    }

    exit 0 ;

}


######################################
# This is the main processsite subroutine:

=pod

=head2 processsite subroutine

This is the main subroutine to process a aww project.

First it checks and executes the options for every directory, then it
processes all other files of the project.

=cut




sub processsite {
    print "-- executing processsite subroutine --\n" if $Aww::verbose > 1 ;
    shift;
    my @ProcessSites = @_;

    my @ExistingSites = checksite(@_);
    print_error("The project(s) '@ProcessSites' don't exist! You have to create a config file first.\n\n") unless @ExistingSites;


    foreach my $ProcessSite (@ExistingSites) {
	my $ProcessFile = "$ProcessSite.aww" ;

	our $ProcessVars = config($ProcessFile) ;

	print "\n\n\n  Start processing ${ProcessSite}: \n\n" if $Aww::verbose ;
#	my @ProcessDirs = @{$ProcessVars->{'Main.ProcessDirectories'}} ;
	my @ProcessDirs = config("$ProcessSite.aww","Main.ProcessDirectories") ;

	foreach my $OptionDir ( @ProcessDirs ) {


	    # Create the matching Output-Directory if not existend
	    my $OutputDir = $OptionDir ;
	    $OutputDir =~ s/$ProcessVars->{'Main.Src_Base_Directory'}/$ProcessVars->{'Main.Output_Base_Directory'}/ ;
	    mkdir $OutputDir unless -d $OutputDir ;



	    # Change to Src Directory

	    chdir $OptionDir or die "Can't change to $OptionDir!\n";


	    # Make sure the Options-part is only run for the Directories that have options!
	    my @OptionsForDir = config($ProcessFile,"Options.$OptionDir") ;


	    print "-- processing directory: $OptionDir\n   with Options: @OptionsForDir\n" if $Aww::verbose ;


	    if ( @OptionsForDir ) {


		# 2. CheckOptions

		checkoptions(@OptionsForDir);

		# 3. Execute Options for each directory

		execoptions($OptionDir,@OptionsForDir);
	    }


	    # 4. WML-Processing the Directories

	    wmlprocess($OptionDir);


	}
	# delete the created-wml files. 
	# Otherwise they may be connected in the next aww run!

	trashfiles("_CLEAR") ;


    }

    return ;

}





1;
__END__


=pod

=head1 SEE ALSO

B<L<aww>>, B<L<Aww::lib>> and B<L<Aww::Options>>

=head1 AUTHOR

Michael Panhorst <aww[at]panhorst.com>

=head1 BUGS

None known.



=cut
