package Aww::Options;

use strict;
use warnings;

use File::Copy;


sub duplicate {
    my $SrcDir = shift ;


    print "-- executing Aww::Options::duplicate subroutine --\n" if $Aww::verbose > 1 ;

    my $CPtype = "" ;
    if ( -r "$SrcDir/.duplicaterc") {

	$CPtype = Aww::lib::config("$SrcDir/.duplicaterc","copytype") if Aww::lib::config("$SrcDir/.duplicaterc","copytype");
    }

    my @SrcFiles = Aww::lib::get_fullfilenames($SrcDir,"none",$CPtype) ;

    my @NewFiles = Aww::lib::checktime(@SrcFiles) ;


# copy all file from SrcDir to DestDir:

    foreach my $SrcFile ( @NewFiles ) {
	next if -d $SrcFile  ;
	my $DestFile = $SrcFile ;
	$DestFile =~ s/^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'}/$Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}/ ;
	copy("$SrcFile","$DestFile") or  die "Cannot copy $SrcFile to $DestFile: $!"  ;
	print "Copying '$SrcFile' to '$DestFile'\n" if $Aww::verbose > 2 ;
    }


#    return $DestDir ;

}




1;
__END__
# Below is the documentation for the module. Keep it up to date!

=head1 NAME

B<Aww::Options::duplicate> - an Aww-Options-Module!

The duplicate module copies all files in the source directory directly
to the output directory (e.g. useful for image directories).

You can define an Regular Expression in a '.duplicaterc' file, which
files should be copied (see below).


=head1 SYNOPSIS

=over 7

=item B<Input :> Directory to duplicate into Output_base_directory +Relative_directory

=item B<Output:> The duplicated directory (of course)

=item B<Return:> The destination directory

=back


=head2 .duplicaterc

You can define an Regular Expression in a '.duplicaterc' file, which
files should be copied. The file should look like this (example to
copy only Images):

##################################################################

# Regular Expression to define the Filetypes, that will be copied:
copytype='\.jpg$|\.gif$|\.png$'

##################################################################

The module searches for the 'copytype' option in the file and uses it for the the getfullfilenames function.

=head1 REQUIREMENTS

B<File::Copy> module is needed


=head1 AUTHOR

Michael Panhorst <aww[at]panhorst.com>

=head1 SEE ALSO

B<L<Aww::Options>>, B<L<Aww::Main>> and B<L<Aww::lib>>

=cut
