package Aww::Options;

use strict;
use warnings;

use HTML::Parser ;
use File::Copy ;




sub index {
    my $Dir = shift ;
    my $IndexVars ;

    chdir $Dir ;

    print "-- executing Aww::Options::index module --\n" if $Aww::verbose > 2 ;


    # Read .indexrc-file if it is in place

    print "   checking config file: $Dir/.indexrc\n" if $Aww::verbose > 2 ;

    if ( -r "$Dir/.indexrc" ) {
	$IndexVars = Aww::lib::config("$Dir/.indexrc") ;

    }

    # Create new .indexrc file otherwise

    else {
	
	my $IndexrcFile = <<'_CONFIG';
########################################################
#      This is the default .indexrc file               #
########################################################
#
# see 'man Aww::Options::index' for more informations!
#
# Always use the '' to make sure the "" of the html works


# This is the main section of the module
[indexpages]   

# Page header and footer of the created indexpage
header='<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">\
<head><title>Index page</title></head>\
<html><body>'
footer='</html></body>'

# What comes before and after every index entry
preentry='<p>'
postentry='</p>'

# What kind of indexpages do you want?
# Possible options are: none, one, alpha, reverse or 
# a number (which is the number of entries per indexpage)
indextype='one'

# This module provides the following sort options:
# alpha   : Alphanumerical case insensitive 
# Alpha   : Alphanumerical case sensitive
# reverse : Alphanumerical reverse (case insensitive)
# Reverse : Alphanumerical reverse (case sensitive)
# time    : newest first
# Time    : Oldest first
sortoption='alpha'

# options of the href tag, e.g. target="_blank"
hrefoptions=''

# Replaces all underscore with a space in the entries
underscorereplace='1'

# looks for a hidden file that prepends this annotate ending 
# (e.g. .SomeFile.wml) and adds the contents 
# of that file to the index entry
annotatefile='.'

# basename of the indexpages:
indexbasename='index'


# RegExp to match the files that should be indexed (e.g. '.wml$' for
# files with the .wml ending. Standard is all (non hidden) files.
filematch=''

# to force the creation of the index-page
forceindex=''




# Options to index pictures in the directory
[pictures]   

# RegExp defines, what Files should be handled as pictures
# ignoring the case for matching
imagetypes='\.jpg$|\.gif$|\.png$'

# if this option is set, a thumbnail will be created
# (thumbs are not supported for animated gifs!)
# The Imageformat of the created thumbs(should be png,gif or jpg)
thumbtype='png'

# maximal height or width (whatever is bigger) of the thumb
maxheight='50'
maxwidth='50'

# name of the created directory to put the thumbs in
thumbdir='thumb'

# options for the <img> tag of the thumb
thumboptions='align="top"'

# options for the <img> tag of the created imagepage
imageoptions=''

# header and footer of the created imagepage
header='<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">\
<html><body><p align="center"><a href="<get-var PREVIOUSFILE>">\
Previous</a>&nbsp;|&nbsp;<a href="<get-var UPFILE>">Index</a>\
&nbsp;|&nbsp;<a href="<get-var NEXTFILE>">Next</a></p><p align="center">'
footer='</p></html></body>'




# For (a little) backward compatibility
[wmlfiles]   

# Defines the Tag to look for in the files with
# Processfile_type Ending

WMLTocTag='short'





# Comments to known file formats. Add whatever you need here!
[known]   

pps='(PowerPoint Presentation)'
ppt='(PowerPoint Presentation)'
doc='(Word Document)'
pdf='(PDF Document)'
ps='(PostScript Document)'
txt='(Textfile)'

mpg='(MPEG Video)'
avi='(AVI Video)'
swf='(Flash Animation)'

wav='(Audio file)'
mp3='(MP3 file)'

xcf='(GIMP graphic)'


# wml='(webpage)'
# jpg='(Image)'
# gif='(Image)'
# png='(Image)'



########################################################

_CONFIG


open (INDEXRC, ">$Dir/.indexrc") or die "***ERROR*** Couldn't create new config file: $Dir/.indexrc \n " ;
print INDEXRC $IndexrcFile ;
close INDEXRC ;

print "Writing new .indexrc file in Directory: $Dir \n"; 
print "Please edit the file for your needs! --> Exiting!\n"; 
exit 2;
}


# Check only unless forceindex is not set:

unless ( $IndexVars->{'indexpages.forceindex'} ) {

    # First get the wanted filenames (already sorted)

    my @CheckList = Aww::lib::get_fullfilenames($Dir,$IndexVars->{'indexpages.sortoption'},$IndexVars->{'indexpages.filematch'}) ;


    # and check if anything needs to be done at all

    my @Check = Aww::lib::checktime(@CheckList) ;

    unless ( @Check ) {
	print "Nothing to be done for $Dir \n" if $Aww::verbose > 1 ;
	return ;
    }

}


# Get all wanted Filenames (without path) to process

my @FileList = Aww::lib::get_filenames($Dir,$IndexVars->{'indexpages.sortoption'},$IndexVars->{'indexpages.filematch'}) ;



#my $PFT = Aww::lib::config("config","Processfile_type") ;
my $PFT = "\.wml" ; # Hardcoded, so that the .swml stuff works
my $OFT = Aww::lib::config("config","Outputfile_type") ;
no warnings ;
$OFT = eval "\"$OFT\"" ;
use warnings ;


my @IndexPlus ;
my @IndexFiles ;
my @ConnectFiles ;
my $EntryCounter = 0 ;
my $NumberTrunc = "-1";



# check what kind of index is wanted

for ( $IndexVars->{'indexpages.indextype'} ) {

    $_ eq "one"   and do {

	@IndexPlus = "" ;
	last ;

    };
    $_ eq "alpha"   and do {

	Aww::lib::print_error("Sortoption must be 'alpha' for indextype='alpha'") unless( $IndexVars->{'indexpages.indextype'} eq 'alpha' ) ;
	@IndexPlus = ("a".."z") ;

	last ;

    };
    $_ eq "reverse"   and do {

	Aww::lib::print_error("Sortoption must be 'reverse' for indextype='reverse'") unless( $IndexVars->{'indexpages.indextype'} eq 'reverse' ) ;
	@IndexPlus = ("a".."z") ;
	@IndexPlus = sort { lc($b) cmp lc($a) } @IndexPlus ;

	last ;

    };
    $_ > 0     and do {

	my $IndexMax = int ($#FileList / $IndexVars->{'indexpages.indextype'}) + 1 ;

	@IndexPlus = (1..$IndexMax) ;
	$NumberTrunc = $IndexVars->{'indexpages.indextype'} ;
	last ;

    };


Aww::lib::print_error("Unknown indextype in $Dir/.indexrc file") ;

}






#############################################
# major index loops


 INDEX: for my $i ( 0 .. $#IndexPlus ) {

    # Set RegExp according to indextype

    my $RegExp ;

    if ( $IndexVars->{'indexpages.indextype'} =~ /^alpha$|^revers$/ ) {
	$RegExp = "^[$IndexPlus[$i]|\\d|_|\\W]" ;
    }
    else{
	$RegExp = ".*" ;	
    }




    $IndexPlus[$i] = "" if $i == 0 ;

    print "\nCreating indexfile: $IndexVars->{'indexpages.indexbasename'}$IndexPlus[$i]$PFT\n" if $Aww::verbose > 1 ; 


open (INDEXFILE, ">$Dir/$IndexVars->{'indexpages.indexbasename'}$IndexPlus[$i]$PFT") or die "Couldn't open indexfile:$Dir/$IndexVars->{'indexpages.indexbasename'}$IndexPlus[$i]$PFT\n" ; 

push @IndexFiles , "$IndexVars->{'indexpages.indexbasename'}$IndexPlus[$i]$PFT" ;

Aww::lib::trashfiles("$Dir/$IndexVars->{'indexpages.indexbasename'}$IndexPlus[$i]$PFT") ;

# print header:
print INDEXFILE qq($IndexVars->{'indexpages.header'}\n) ;




  while ( $FileList[$EntryCounter] =~ /$RegExp/i ) {

      print "Checking filetype etc...\nFile: $FileList[$EntryCounter]\nCounter: $EntryCounter von $#FileList\n " if $Aww::verbose > 2 ;



      my $PrintFile ;
      my $LinkDest ;
      my $AdditionalText = "" ;
      my $hrefimg = "" ;
      my $Explanation = "" ;

      # Check if the File is a symbolic link

      if ( -l $FileList[$EntryCounter] ) {

	  $PrintFile = $FileList[$EntryCounter] ;
	  $LinkDest = readlink $FileList[$EntryCounter] ;

	  my $TryOutput = "$Dir/$LinkDest" ;
	  $TryOutput =~ s(^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'})($Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}) ;
	  $TryOutput =~ s/\.\w*$/$OFT/ ;

	  if ( -r $TryOutput ) {
	      $LinkDest =~ s/\.\w*$/$OFT/ ;
	      $PrintFile =~ s/\.\w*$// ;
          }

	  # check for underscorereplacement of the filenames
	  $PrintFile =~ tr/_/ / if $IndexVars->{'indexpages.underscorereplace'} ;
      }



      # Check if File is a wml file 

      elsif ( $FileList[$EntryCounter] =~ /$PFT$/ ) {

	  push @ConnectFiles , $FileList[$EntryCounter] ;
	  

	  open (WMLFILE, "<$Dir/$FileList[$EntryCounter]") or die "Couldn't open $FileList[$EntryCounter] while processing $Dir !\n" ;


        our $Blafaselirgendwaslanges = $IndexVars->{'wmlfiles.WMLTocTag'} ;
	# subroutine after found initial $TocTag:
	sub start_handler {
	    return if shift ne "$Blafaselirgendwaslanges";
	    my $self = shift;
	    $self->handler(text => sub { our $Short = shift }, "dtext");
	    $self->handler(end  => sub { shift->eof if shift eq "$Blafaselirgendwaslanges"; },
			   "tagname,self");
	}	
	  # Create new HTML::Parser Object
	  my $Extract = HTML::Parser->new(api_version => 3);
	  # Start handler (what to do after <$TocTag>
	  $Extract->handler( start => \&start_handler, "tagname,self");
	  # Parse the open file:
	  $Extract->parse_file(*WMLFILE) || die $!;

	  close WMLFILE ;	  

	  $AdditionalText = "<BR>\n $Aww::Options::Short" if $Aww::Options::Short ;
	  $Aww::Options::Short = "" ; # clear Variable 
	  
	  $LinkDest = $FileList[$EntryCounter] ;
	  $PrintFile = $LinkDest ;
	  $PrintFile =~ s/$PFT$// ;
	  # check for underscorereplacement of the filenames
	  $PrintFile =~ tr/_/ / if $IndexVars->{'indexpages.underscorereplace'} ;
	  $LinkDest = Aww::lib::pft2oft($LinkDest) ;


      }
      


      # Check if file is a picture

      elsif ( $FileList[$EntryCounter] =~ /$IndexVars->{'pictures.imagetypes'}$/i ) {

	  $LinkDest = $FileList[$EntryCounter] ;
	  $PrintFile = $FileList[$EntryCounter] ;
	  my $ImgFile = $FileList[$EntryCounter] ;

	  $PrintFile =~ s/$IndexVars->{'pictures.imagetypes'}$//;
	  $PrintFile =~ tr/_/ / if $IndexVars->{'indexpages.underscorereplace'} ;	      

	  $ImgFile =~ s/\.\w*$/$PFT/ ;

	  $LinkDest =~ s/\.\w*$/$OFT/ ;


	  # Creating wmlfile for the image:

	  open (IMGFILE, ">$Dir/$ImgFile") or die "Couldn't open $ImgFile while processing $Dir !\n" ;
	  Aww::lib::trashfiles("$Dir/$ImgFile") ;

	  print IMGFILE qq($IndexVars->{'pictures.header'}) ;

	  print IMGFILE qq(<IMG SRC="$FileList[$EntryCounter]" alt="$PrintFile" $IndexVars->{'pictures.imageoptions'}> ) ;
	  print IMGFILE qq(<BR>$PrintFile) ;
	  print IMGFILE qq($IndexVars->{'pictures.footer'}) ;
	  close IMGFILE ;

	  # adding to Connectfiles

	  push @ConnectFiles , $ImgFile ;



	  # creating thumb and copying Image only if necessary and wanted

	  my $SrcFile = "$Dir/$FileList[$EntryCounter]" ;
	  if ( Aww::lib::checktime($SrcFile) ) {
	      


	      if ( $IndexVars->{'pictures.thumbtype'} ) {

		  Aww::lib::createthumb($Dir,$IndexVars->{'pictures.thumbtype'},$IndexVars->{'pictures.thumbdir'},$IndexVars->{'pictures.maxwidth'},$IndexVars->{'pictures.maxheight'},$FileList[$EntryCounter]) ;	


		  my $Thumblink = "$IndexVars->{'pictures.thumbdir'}/$FileList[$EntryCounter]" ;
		  $Thumblink =~ s(\.\w*$)(\.$IndexVars->{'pictures.thumbtype'}) ;
		  $hrefimg = qq(<img src="$Thumblink" alt="$PrintFile" $IndexVars->{'pictures.thumboptions'}>) ;      
		  
	      }
	      my $DestFile = $SrcFile ;
	      $DestFile =~ s(^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'})($Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}) ;
	      copy("$SrcFile","$DestFile") or  die "Cannot copy $SrcFile to $DestFile: $!"  ;
	      print "Copying '$SrcFile' to '$DestFile'\n" if $Aww::verbose > 2 ;		  

	  }




      }
      else {
	  $LinkDest = $FileList[$EntryCounter] ;
	  $PrintFile = $LinkDest ;


	  # check for underscorereplacement of the filenames
	  $PrintFile =~ tr/_/ / if $IndexVars->{'indexpages.underscorereplace'} ;

	  my $SrcFile = "$Dir/$FileList[$EntryCounter]" ;

	  if ( Aww::lib::checktime($SrcFile) ) {

	      my $DestFile = $SrcFile ;
	      $DestFile =~ s(^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'})($Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}) ;
	      copy("$SrcFile","$DestFile") or  die "Cannot copy $SrcFile to $DestFile: $!"  ;
	      print "Copying '$SrcFile' to '$DestFile'\n" if $Aww::verbose > 2 ;
	  }
      }


	  # check for file explanation [known]

	  my $CheckExplain = lc($FileList[$EntryCounter]) ;
	  $CheckExplain =~ s/.*\.// ;
	  $Explanation = Aww::lib::config("$Dir/.indexrc","known.$CheckExplain") if Aww::lib::config("$Dir/.indexrc","known.$CheckExplain") ;

	  $PrintFile =~ s/\.$CheckExplain$// if $Explanation ;





      # check for an annotationfile

      if ( -r "$Dir/$IndexVars->{'indexpages.annotatefile'}$FileList[$EntryCounter]" ) {

	  my $Annotation = qx(cat $Dir/$IndexVars->{'indexpages.annotatefile'}$FileList[$EntryCounter]) ;
	  $AdditionalText = "$AdditionalText $Annotation" ;

      }




      print INDEXFILE qq($IndexVars->{'indexpages.preentry'}\n);
      print INDEXFILE qq(<a href="$LinkDest" $IndexVars->{'indexpages.hrefoptions'}>$hrefimg$PrintFile</a> $Explanation\n) ;
      print INDEXFILE qq($AdditionalText\n) ;
      print INDEXFILE qq($IndexVars->{'indexpages.postentry'}\n\n) ;
      




      # if all entries are done
      if ( $EntryCounter== $#FileList ) {
	  print INDEXFILE qq($IndexVars->{'indexpages.footer'}\n) ;
	  close INDEXFILE ;
	  next INDEX ;
      }      

      $EntryCounter++ ;

      if ( $NumberTrunc == $EntryCounter ) {
	  $NumberTrunc = $NumberTrunc + $IndexVars->{'indexpages.indextype'} ;
	  print INDEXFILE qq($IndexVars->{'indexpages.footer'}\n) ;
	  close INDEXFILE ;
	  next INDEX ;
      }


  }


print INDEXFILE qq($IndexVars->{'indexpages.footer'}\n) ;
close INDEXFILE ;

$i++ ;

}

Aww::lib::connect($Dir,$IndexVars->{'indexpages.indextype'},$IndexVars->{'indexpages.indexbasename'},@ConnectFiles) ;

Aww::lib::connect($Dir,'none','dontcare',@IndexFiles) ;



return ;
    
}

1;
__END__





# Below is the documentation for the module. Keep it up to date!

=head1 NAME 

B<Aww::Options::index> is an aww options module!

The index module is indexing all files in a directory. The behaviour
of the module can be changed with several options in the .indexrc
file. The old toc, wmltoc and new modules were put into this new
module.

=head1 DESCRIPTION

All Options of the .indexrc file are described below:

=head2 [indexpages]   

=over 7

=item B<header:> header for the created indexpage(s)

=item B<footer:> footer for the created indexpage(s)

=item B<preentry:> stuff to put before every indexentry

=item B<postentry:> stuff to put after every indexentry

=item B<indextype:> possible indextypes are: none, one,alpha, reverse or a number. B<none> creates no indexpage and just connects the files, B<one> puts all entries on one indexpage, B<alpha> creates one indexpage per letter - all files starting with 'a' are put on indexa.html ..., B<reverse> is alpha reversed and if set to a number, it's the number of entries per indexpage.

=item B<sortoption:> the following sort options are supported: alpha -  Alphanumerical case insensitive, Alpha - Alphanumerical case sensitive, reverse - Alphanumerical reverse (case insensitive), Reverse - Alphanumerical reverse (case sensitive), time - newest image first, Time - Oldest image first

=item B<hrefoptions:> special options for the <a href..> tag of the link to the indexentry

=item B<underscorereplace:> if set, all underscores are replaced with a space in the labels (see abovelabel and belowlabel

=item B<annotatefile:>  looks for a hidden file that prepends this annotate ending (e.g. '.SomeFile.wml') and adds the contents of that file to the entry on the indexpage

=item B<indexbasename:> the basename of the created indexfile

=item B<filematch:> RegExp to match the files that should be indexed (e.g. '.wml$' for files with the .wml ending. If not set, all (non hidden) files are indexed.

=back


=head2 [pictures]

=over 7

=item B<imagetypes:> RegExp defines, what files are handled as pictures by this module

=item B<thumbtype:> Imageformat of the created thumbnails (should be png,gif or jpg), only if this option is set, a thumbnail will be created

=item B<maxwidth:> maximum width (in pixel) of the thumbnail to create

=item B<maxheight:> maximum height (in pixel) of the thumbnail to create

=item B<thumbdir:> name of the directory to put the created thumbnails in

=item B<thumboptions:> special options for the <img> tag of the created thumbnail entry

=item B<imageoptions:> special options for the <img> tag of the created imagepage

=item B<header:> header for the created imagepage

=item B<footer:> footer for the created imagepage


=back

=head2 [wmlfiles]

=over 7

=item B<WMLTocTag:> Defines the Tag to look for in the files with Processfile_type Ending (like in the old wmltoc module)

=back

=head2 [known]

Comments to known file formats. Add whatever you need here! These comments are added besides the link. If you don't like that, comment all out. If you need more just add them under this section.


=head1 SYNOPSIS

=over 7

=item B<Input :> <Directory to process>

=item B<Output:> index wml-file

=back


=head1 REQUIREMENTS

The B<HTML::Parser>,B<Image::Magick> and B<File::Copy> modules are
needed to use this module

=head1 AUTHOR

Michael Panhorst <aww[at]panhorst.com>

=head1 SEE ALSO

B<L<Aww::Options>>, B<L<Aww::Main>> and B<L<Aww::lib>>

=cut
