package Aww::Options;

use strict;
use warnings;
use File::Copy;


sub thumbnail {
    my $Dir = shift ;
    my $ThumbnailVars ;

    print "-- executing Aww::Options::thumbnail subroutine --\n" if $Aww::verbose > 2 ;

    print "   checking config file: $Dir/.thumbnailrc\n" if $Aww::verbose > 2 ;

    if ( -r "$Dir/.thumbnailrc" ) {
	$ThumbnailVars = Aww::lib::config("$Dir/.thumbnailrc") ;

    }

    # Create new .indexrc file otherwise

    else {
	
	my $ThumbnailrcFile = <<'_CONFIG';
########################################################
#      This is the default .thumbnailrc file           #
########################################################
#
# see 'man Aww::Options::thumbnail' for more information
#
# Always use the '' to make sure the "" of the html works


# ------------------------------------------------------
# options for the thumbnail creation
[thumbnail]

# the Imageformat of the created thumbs (should be png,gif or jpg)
type='png'

# name of the created directory to put the thumbs in
dir='thumb'

# maximal height or width (whatever is bigger) of the thumb
maxwidth='100'
maxheight='100'



# ------------------------------------------------------
# options for creating the thumbnail pages
[thumbpages]

# header and footer of the created thumbnailpage(s)
header='<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">\
<html><body><p align="center"><a href="<get-var PREVIOUSFILE>">\
Previous</a>&nbsp;|&nbsp;<a href="<get-var NEXTFILE>">Next</a></p>\
<div align="center">'
footer='</div></html></body>'

# possible types for the thumbnailpage(s) are:
# standard : for normal columns x rows thumbnailpages
# one      : to get all thumbnails on one page 
#            (only columns is evaluated)
# alpha    : one thumbnailpage for every letter 
#            (only columns is evaluated)
# reverse  : alpha sorted reverse (see sortoption)
type='standard'

# This module provides the following sort options:
# alpha   : Alphanumerical case insensitive
# Alpha   : Alphanumerical case sensitive
# reverse : Alphanumerical reverse (case insensitive)
# Reverse : Alphanumerical reverse (case sensitive)
# time    : newest first
# Time    : Oldest first
sortoption='alpha'



# number of columns and rows for the thumbnailpage(s)
columns='4'
rows='4'

# thumbnail-label (above or below the thumbnail)
# possibilities are:
# %f : full filename
# %n : filename without extension
# %h : height of the original picture
# %w : width of the original picture
# %b : Filesize (in byte)
# %k : Filesize (in kilobyte)
abovelabel=''
belowlabel='<BR>%n<BR>(%wx%hpx)<BR>(%kkb)'


# options for the created table
# the module is using the wml-module grid
# (see 'man wml::std::grid' for more infos)
gridoptions='spacing="5" summary="Image overview"'
celloptions='align="center" valign="middle"'

# options of the href tag, e.g. target="_blank"
hrefoptions=''

# options for the IMG-tag of the thumbnail
imgoptions=''


# replaces all underscore with a space in the entries
# (as well for the picture-pages)
underscorereplace='1'

# basename of the thumbnailpage(s):
indexbasename='index'



# ------------------------------------------------------
# Options for creating the picture-pages
[picturepages]


# header and footer of the created picturepages
header='<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">\
<html><body><p align="center"><a href="<get-var PREVIOUSFILE>">\
Previous</a>&nbsp;|&nbsp;<a href="<get-var UPFILE>">Index</a>\
&nbsp;|&nbsp;<a href="<get-var NEXTFILE>">Next</a></p><p align="center">'
footer='</p></html></body>'

# RegExp defining, what files are handled by this module
# ignoring the case for matching
imagetypes='\.jpg$|\.gif$|\.png$'

# picture-label (above or below the thumbnail)
# possibilities are:
# %f : full filename
# %n : filename without extension
# %h : height of the original picture
# %w : width of the original picture
# %b : Filesize (in byte)
# %k : Filesize (in kilobyte)
abovelabel=''
belowlabel='<BR>%n'

# options for the IMG-tag of the picture
imgoptions=''

# if this option is set, the link goes directly to 
# the Imagefile, and you may want to add an
# 'target="_blank"' to the hrefoptions above 
# no html files for the pictures will be created
nohtml=



########################################################

_CONFIG


open (THUMBNAILRC, ">$Dir/.thumbnailrc") or die "***ERROR*** Couldn't create new config file: $Dir/.thumbnailrc \n " ;
print THUMBNAILRC $ThumbnailrcFile ;
close THUMBNAILRC ;

print "*** Writing new .thumbnailrc file in Directory: $Dir \n"; 
print "--> Please edit the file for your needs and run aww again!\n" ;
return ;
}


# get the wanted filenames (already sorted)

my @CheckList = Aww::lib::get_fullfilenames($Dir,$ThumbnailVars->{'thumbpages.sortoption'},$ThumbnailVars->{'picturepages.imagetypes'}) ;


# and check if anything needs to be done at all

my @Check = Aww::lib::checktime(@CheckList) ;


unless ( @Check ) {
    print "Nothing to be done for $Dir \n" if $Aww::verbose > 1 ;
    return ;
}


# 1. copy image and create thumb only for the new files

foreach my $NewFile ( @Check ) {

    my $DestFile = $NewFile ;
    $DestFile =~ s(^$Aww::Main::ProcessVars->{'Main.Src_Base_Directory'})($Aww::Main::ProcessVars->{'Main.Output_Base_Directory'}) ;
    copy("$NewFile","$DestFile") or  die "Cannot copy $NewFile to $DestFile: $!"  ;
    print "Copying '$NewFile' to '$DestFile'\n" if $Aww::verbose > 2 ;

    $NewFile =~ s($Dir/)() ;

}

Aww::lib::createthumb($Dir,$ThumbnailVars->{'thumbnail.type'},$ThumbnailVars->{'thumbnail.dir'},$ThumbnailVars->{'thumbnail.maxwidth'},$ThumbnailVars->{'thumbnail.maxheight'},@Check) ;







# Get all wanted Filenames (without path) to process

my @FileList = Aww::lib::get_filenames($Dir,$ThumbnailVars->{'thumbpages.sortoption'},$ThumbnailVars->{'picturepages.imagetypes'}) ;



my $OFT = Aww::lib::config("config","Outputfile_type") ;
#my $PFT = Aww::lib::config("config","Processfile_type") ;
my $PFT = "\.wml" ;  # hardcoded, so that the .swml stuff works
no warnings ;
$OFT = eval "\"$OFT\"" ;
use warnings ;





# 2. Create thumbpages

my @ThumbnailPlus ;
my @ThumbnailFiles ;
my $NumberTrunc = "-1" ;
my $EntryCounter = 0 ;
my $Rows ;
my %AlphaCounter ;

# check what kind of index is wanted

for ( $ThumbnailVars->{'thumbpages.type'} ) {


    $_ eq "standard"     and do {

	$NumberTrunc = $ThumbnailVars->{'thumbpages.columns'} * $ThumbnailVars->{'thumbpages.rows'} ;
	my $IndexMax = int ($#FileList / $NumberTrunc) + 1 ;
	@ThumbnailPlus = (1..$IndexMax) ;
	$Rows = $ThumbnailVars->{'thumbpages.rows'} ;
	last ;

    };
    $_ eq "one"   and do {

	@ThumbnailPlus = "" ;
	$Rows = int($#FileList / $ThumbnailVars->{'thumbpages.columns'}) +1 ;

	last ;

    };
    $_ eq "alpha"   and do {

	Aww::lib::print_error("Sortoption must be 'alpha' for type='alpha'") unless( $ThumbnailVars->{'thumbpages.type'} eq 'alpha' ) ;

	@ThumbnailPlus = ("a".."z") ;

	for ( "a".."z" ) {
	    $AlphaCounter{$_} =  "0" ;

	}

	  my @CountList = @FileList ;
	  foreach ( @CountList ) {
	      $_ = lc(substr($_,0,1)) ;
	      $_ = "a" unless $_ =~ /[a-z]/ ;
	      $AlphaCounter{$_}++ ;
	  }

	last ;

    };
    $_ eq "reverse"   and do {

	Aww::lib::print_error("Sortoption must be 'reverse' for type='reverse'") unless( $ThumbnailVars->{'thumbpages.type'} eq 'reverse' ) ;
	@ThumbnailPlus = ("a".."z") ;
	@ThumbnailPlus = sort { lc($b) cmp lc($a) } @ThumbnailPlus ;


	for ( "a".."z" ) {
	    $AlphaCounter{$_} =  "0" ;

	}

	  my @CountList = @FileList ;
	  foreach ( @CountList ) {
	      $_ = lc(substr($_,0,1)) ;
	      $_ = "z" unless $_ =~ /[a-z]/ ;
	      $AlphaCounter{$_}++ ;
	  }

	last ;

    };



Aww::lib::print_error("Unknown type in $Dir/.thumbnailrc file") ;

}






#############################################
# Thumbnailpage creation


 THUMBNAIL: for my $i ( 0 .. $#ThumbnailPlus ) {

    # Set RegExp according to indextype

    my $RegExp ;

    if ( $ThumbnailVars->{'thumbpages.type'} =~ /^alpha$|^reverse$/ ) {
	$RegExp = "^[$ThumbnailPlus[$i]|\\d|_|\\W]" ;
    }
    else{
	$RegExp = ".*" ;	
    }

    # Set $Rows for alpha or reverse

    if ( $ThumbnailVars->{'thumbpages.type'} =~ /^alpha$|^reverse$/ ) {

	$Rows = int( $AlphaCounter{$ThumbnailPlus[$i]} / $ThumbnailVars->{'thumbpages.columns'} ) +1 ;

    }


    $ThumbnailPlus[$i] = "" if $i == 0 ;

    print "\nCreating thumbnailfile: $ThumbnailVars->{'thumbpages.indexbasename'}$ThumbnailPlus[$i]$PFT\n" if $Aww::verbose > 1 ; 


open (THUMBNAILFILE, ">$Dir/$ThumbnailVars->{'thumbpages.indexbasename'}$ThumbnailPlus[$i]$PFT") or die "Couldn't open thumbnailfile:$Dir/$ThumbnailVars->{'thumbpages.indexbasename'}$ThumbnailPlus[$i]$PFT\n" ; 

push @ThumbnailFiles , "$ThumbnailVars->{'thumbpages.indexbasename'}$ThumbnailPlus[$i]$PFT" ;

Aww::lib::trashfiles("$Dir/$ThumbnailVars->{'thumbpages.indexbasename'}$ThumbnailPlus[$i]$PFT") ;




# print header:
print THUMBNAILFILE qq(\#use wml::std::grid \n) ;
print THUMBNAILFILE qq($ThumbnailVars->{'thumbpages.header'} \n) ;
print THUMBNAILFILE qq(<grid layout="$ThumbnailVars->{'thumbpages.columns'}x$Rows" $ThumbnailVars->{'thumbpages.gridoptions'}> \n) ;



  while ( $FileList[$EntryCounter] =~ /$RegExp/i ) {


	my $LinkDest = $FileList[$EntryCounter] ;
	$LinkDest =~ s/$ThumbnailVars->{'picturepages.imagetypes'}/$OFT/ unless $ThumbnailVars->{'picturepages.nohtml'} ;
	my $Basename = $FileList[$EntryCounter] ;
	$Basename =~ s/$ThumbnailVars->{'picturepages.imagetypes'}// ;
	my $Properties = Aww::lib::image_properties("$Dir/$FileList[$EntryCounter]") ;

	my $AboveThumb = $ThumbnailVars->{'thumbpages.abovelabel'} ;
	$AboveThumb =~ s/\%f/$FileList[$EntryCounter]/g ;
	$AboveThumb =~ s/\%n/$Basename/g ;
	$AboveThumb =~ tr/_/ / if $ThumbnailVars->{'thumbpages.underscorereplace'} ;
	$AboveThumb =~ s/\%h/$Properties->{'height'}/g ;
	$AboveThumb =~ s/\%w/$Properties->{'width'}/g ;
	$AboveThumb =~ s/\%b/$Properties->{'size'}/g ;
	$AboveThumb =~ s/\%k/$Properties->{'ksize'}/g ;
	
	
	my $BelowThumb = $ThumbnailVars->{'thumbpages.belowlabel'} ; 
	$BelowThumb =~ s/\%f/$FileList[$EntryCounter]/g ;
	$BelowThumb =~ s/\%n/$Basename/g ;
	$BelowThumb =~ tr/_/ / if $ThumbnailVars->{'thumbpages.underscorereplace'} ;
	$BelowThumb =~ s/\%h/$Properties->{'height'}/g ;
	$BelowThumb =~ s/\%w/$Properties->{'width'}/g ;
	$BelowThumb =~ s/\%b/$Properties->{'size'}/g ;
	$BelowThumb =~ s/\%k/$Properties->{'ksize'}/g ;




      print THUMBNAILFILE qq(<cell $ThumbnailVars->{'thumbpages.celloptions'}> \n) ;
      print THUMBNAILFILE qq($AboveThumb \n) ;
      print THUMBNAILFILE qq(<a href="$LinkDest" $ThumbnailVars->{'thumbpages.hrefoptions'}><img src="$ThumbnailVars->{'thumbnail.dir'}/$Basename.$ThumbnailVars->{'thumbnail.type'}" alt="$Basename" $ThumbnailVars->{'thumbpages.imgoptions'}></a>\n) ;
      print THUMBNAILFILE qq($BelowThumb \n) ;

      print THUMBNAILFILE qq(</cell> \n) ;







      # if all entries are done
      if ( $EntryCounter == $#FileList ) {
	  print THUMBNAILFILE qq(</grid> \n) ;
	  print THUMBNAILFILE qq($ThumbnailVars->{'thumbpages.footer'} \n) ;
	  close THUMBNAILFILE ;
	  next THUMBNAIL ;
      }


      $EntryCounter++ ;

      if ( $NumberTrunc == $EntryCounter ) {
	  $NumberTrunc = $NumberTrunc + $ThumbnailVars->{'thumbpages.columns'} * $ThumbnailVars->{'thumbpages.rows'} ;
	  print THUMBNAILFILE qq(</grid> \n) ;
	  print THUMBNAILFILE qq($ThumbnailVars->{'thumbpages.footer'} \n) ;
	  close THUMBNAILFILE ;
	  next THUMBNAIL ;
      }


  }


print THUMBNAILFILE qq(</grid> \n) ;
print THUMBNAILFILE qq($ThumbnailVars->{'thumbpages.footer'} \n) ;
close THUMBNAILFILE ;

$i++ ;


}



Aww::lib::connect($Dir,'none','dontcare',@ThumbnailFiles) ;














# 3. Create picturepages (unless nohtml is set)

unless ( $ThumbnailVars->{'picturepages.nohtml'} ) {


    my @ConnectFiles ;
    my $ConnectType ;


    foreach my $ProcessFile ( @FileList ) {

	my $WMLfile = $ProcessFile ;
	$WMLfile =~ s/$ThumbnailVars->{'picturepages.imagetypes'}/$PFT/ ;
	my $Basename = $ProcessFile ;
	$Basename =~ s/$ThumbnailVars->{'picturepages.imagetypes'}// ;
	my $Properties = Aww::lib::image_properties("$Dir/$ProcessFile") ;

	my $AboveImage = $ThumbnailVars->{'picturepages.abovelabel'} ;
	$AboveImage =~ s/\%f/$ProcessFile/g ;
	$AboveImage =~ s/\%n/$Basename/g ;
	$AboveImage =~ tr/_/ / if $ThumbnailVars->{'thumbpages.underscorereplace'} ;
	$AboveImage =~ s/\%h/$Properties->{'height'}/g ;
	$AboveImage =~ s/\%w/$Properties->{'width'}/g ;
	$AboveImage =~ s/\%b/$Properties->{'size'}/g ;
	$AboveImage =~ s/\%k/$Properties->{'ksize'}/g ;
	
	
	my $BelowImage = $ThumbnailVars->{'picturepages.belowlabel'} ; 
	$BelowImage =~ s/\%f/$ProcessFile/g ;
	$BelowImage =~ s/\%n/$Basename/g ;
	$BelowImage =~ tr/_/ / if $ThumbnailVars->{'thumbpages.underscorereplace'} ;
	$BelowImage =~ s/\%h/$Properties->{'height'}/g ;
	$BelowImage =~ s/\%w/$Properties->{'width'}/g ;
	$BelowImage =~ s/\%b/$Properties->{'size'}/g ;
	$BelowImage =~ s/\%k/$Properties->{'ksize'}/g ;




	open (WMLFILE, ">$Dir/$WMLfile") or die "Couldn't create file $Dir/$ProcessFile \n " ;

	print "\nCreating Picturefile: $Dir/$WMLfile \n" if $Aww::verbose > 1 ; 
	Aww::lib::trashfiles("$Dir/$WMLfile") ;


	print WMLFILE "$ThumbnailVars->{'picturepages.header'}" ;
	print WMLFILE "$AboveImage";
	print WMLFILE "<IMG SRC=\"$ProcessFile\" $ThumbnailVars->{'picturepages.imgoptions'}>" ;
	print WMLFILE "$BelowImage";
	print WMLFILE "$ThumbnailVars->{'picturepages.footer'}" ;

	close WMLFILE ;	



	push @ConnectFiles , $WMLfile ;

    }

# Connect the Picturefiles:

if ( $ThumbnailVars->{'thumbpages.type'} =~ /^standard$/ ) {
    $ConnectType = $ThumbnailVars->{'thumbpages.columns'} * $ThumbnailVars->{'thumbpages.rows'} ; 
}
else {
    $ConnectType = $ThumbnailVars->{'thumbpages.type'} ;
}


Aww::lib::connect($Dir,$ConnectType,$ThumbnailVars->{'thumbpages.indexbasename'},@ConnectFiles) ;

}










return ;

}

1;
__END__


# Below is the documentation for the module. Keep it up to date!

=head1 NAME

B<Aww::Options::thumbnail> is an aww options module!

The thumbnail module creates thumbnail-pages for a directory
containing pictures (png's, jpeg's and gif's, the standard webgraphics
are supported) and a wml file for every picture. Then the module
connects the thumbnail-pages and the wml-files.

The module creates a .thumbnailrc file in the directory to configure the
behaviour of the module in this directory.


=head1 DESCRIPTION

All Options of the .thumbnailrc file are described below:

=head2 [thumbnail]

=over 7

=item B<type:> the Imageformat of the created thumbs - should be png,gif or jpg

=item B<dir:> name of the directory to put the created thumbnails in

=item B<maxwidth:> maximum width (in pixel) of the thumbnail to create

=item B<maxheight:> maximum height (in pixel) of the thumbnail to create

=back


=head2 [thumbpages]

=over 7

=item B<header:> header for the created thumbnailpage

=item B<footer:> footer for the created thumbnailpage

=item B<type:> possible types for the thumbnailpage(s) are: standard, one, alpha or reverse. B<standard> gives you normal rows x columns thumbnailpages for the images in the directory. B<one>makes sure that all images are put on one thumbnailpage - only the columns option is evaluated. B<alpha> creates a thumbnailpage for every letter, so all files starting with 'a' go to thumbnailpage indexa.html ... reverse is a reverse alpha. For alpha and reverse the sortoption must be the same.

=item B<sortoption:> supported sortoptions are: alpha -  Alphanumerical case insensitive, Alpha - Alphanumerical case sensitive, reverse - Alphanumerical reverse (case insensitive), Reverse - Alphanumerical reverse (case sensitive), time    - newest image first, Time - Oldest image first

=item B<columns:> number of columns for the thumbnailpage

=item B<rows:> number of rows for the thumbnailpage

=item B<abovelabel:> label to write above every thumbnail image. The following special characters are supported:

%f : full filename

%n : filename without extension

%h : height of the original picture

%w : width of the original picture

%b : Filesize (in byte)

%k : Filesize (in kilobyte)


=item B<belowlabel:> label to write below every thumbnail image. For special characters see abovelabel.

=item B<gridoptions:> options for the created table. Because the table is created with the grid-tag, see 'man wml::std::grid' for more infos on possible options 

=item B<celloptions:> options for the <cell> tag of the created table - see gridoptions.

=item B<hrefoptions:> special options for the <a href..> tag of the link to the pictures

=item B<imgoptions:> special options for the <img> tag of the thumbnail

=item B<underscorereplace:> if set, all underscores are replaced with a space in the labels (see abovelabel and belowlabel)

=item B<indexbasename:> the basename of the created thumbnailpage(s)

=back


=head2 [picturepages]

=over 7

=item B<header:> header for the created picturepage

=item B<footer:> footer for the created picturepage

=item B<imagetypes:> This is the RegExp that defines what files are processed by the thumbnail module. Normally you wouldn't want to change it.

=item B<abovelabel:> label to write above every thumbnail image. For special characters see above (abovelabel of thumbnailpages).

=item B<belowlabel:> label to write below every thumbnail image. For special characters see above.

=item B<imgoptions:> special options for the <img> tag of the image

=item B<nohtml:> if this option is set, the link goes directly to the Imagefile, and you may want to add a 'target="_blank"' to the hrefoptions above no html files for the pictures will be created

=back


=head1 EXAMPLES

Sorry, nothing here yet :-(






=head1 SYNOPSIS

=over 7

=item B<Input :> <Directory to process>

=item B<Output:> <wml-thumbnail file> + [wml-File for every Picture] + connecting the wml files

=back


=head1 REQUIREMENTS

The Image::Magick module and the File::Copy module is needed
to use this module.


=head1 AUTHOR

Michael Panhorst <aww[at]panhorst.com>


=head1 SEE ALSO

L<Aww::Options>, L<Aww::Main>, L<Aww::lib>

=cut
